/*
 * File:   main18.c
 * Author: ppotty
 *
 * Created on November 26, 2015, 10:28 AM
 */

// CONFIG1
#pragma config FOSC = ECH       // Oscillator Selection (ECH, External Clock, High Power Mode (4-32 MHz): device clock supplied to CLKIN pin)
#pragma config WDTE = OFF       // Watchdog Timer Enable (WDT disabled)
#pragma config PWRTE = OFF      // Power-up Timer Enable (PWRT disabled)
#pragma config MCLRE = ON       // MCLR Pin Function Select (MCLR/VPP pin function is MCLR)
#pragma config CP = OFF         // Flash Program Memory Code Protection (Program memory code protection is disabled)
#pragma config CPD = OFF        // Data Memory Code Protection (Data memory code protection is disabled)
#pragma config BOREN = OFF      // Brown-out Reset Enable (Brown-out Reset disabled)
#pragma config CLKOUTEN = OFF   // Clock Out Enable (CLKOUT function is disabled. I/O or oscillator function on the CLKOUT pin)
#pragma config IESO = ON        // Internal/External Switchover (Internal/External Switchover mode is enabled)
#pragma config FCMEN = ON       // Fail-Safe Clock Monitor Enable (Fail-Safe Clock Monitor is enabled)

// CONFIG2
#pragma config WRT = OFF        // Flash Memory Self-Write Protection (Write protection off)
#pragma config PLLEN = OFF      // PLL Enable (4x PLL disabled)
#pragma config STVREN = ON      // Stack Overflow/Underflow Reset Enable (Stack Overflow or Underflow will cause a Reset)
#pragma config BORV = LO        // Brown-out Reset Voltage Selection (Brown-out Reset Voltage (Vbor), low trip point selected.)
#pragma config LVP = OFF        // Low-Voltage Programming Enable (High-voltage on MCLR/VPP must be used for programming)

#define _XTAL_FREQ 10000000

#include <xc.h>

void adc_init() {
    ADCON0 = 0x0D;
    ADCON1 = 0x40;
}

unsigned long get_rand_seed() {
    char seed_byte = 0;
    unsigned long seed = 0;
    
    while (seed_byte < 4) {
        GO_nDONE = 1;
        while (GO_nDONE);
        seed |= ADRESL << (seed_byte * 8);
        seed_byte++;
    }
    return seed;
}

void main(void) {
    unsigned long rand;
    char hold = 0;
    
    TRISC0 = 0;     //Rock
    TRISC1 = 0;     //Paper
    TRISC2 = 0;     //Scissors
    TRISC4 = 1;     //Activation from RasPi
    ANSC3 = 0;      //analog input for random seeding
    
    adc_init();
    rand = get_rand_seed();
    
    while (1) {
        if (RC4 == 1) {
            if (!hold) {
                
                rand ^= rand << 13;  //Xorshift RNG
                rand ^= rand >> 17;
                rand ^= rand << 5;
                
                if (rand < 1431655765) {
                    RC2 = 1;
                }
                if (rand > 1431655765) {
                    if (rand < 2863311530) {
                        RC1 = 1;
                    }
                }
                if (rand > 2863311530) {
                    RC0 = 1;
                }
                hold = 1;
            }
        }
        else {
            RC0 = 0;
            RC1 = 0;
            RC2 = 0;
            hold = 0;
        }
    }
    return;
}
